/*
   orxonox - the future of 3D-vertical-scrollers

   Copyright (C) 2004 orx

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   ### File Specific
   main-programmer: Patrick Boenzli
   co-programmer: Benjamin Grauer

*/


#define DEBUG_SPECIAL_MODULE DEBUG_MODULE_WEAPON

#include "mbolt.h"

#include "state.h"
#include "model.h"

#include "particles/dot_emitter.h"
#include "particles/box_emitter.h"
#include "particles/sprite_particles.h"

#include "space_ships/space_ship.h"

#include <cassert>
#include "debug.h"

#include "static_model.h"


#include "class_id_DEPRECATED.h"
ObjectListDefinition(MBolt);
CREATE_FAST_FACTORY_STATIC(MBolt);

/**
 *  standard constructor
*/
MBolt::MBolt () : Projectile()
{
  this->registerObject(this, MBolt::_objectList);
  this->loadModel("models/projectiles/mbolt.obj");
  
  //this->loadModel("models/projectiles/laser.obj");

  this->setMinEnergy(4);
  this->setHealthMax(0);
  this->lifeSpan = 5.0;
  this->angle     = 0;


  //this->emitter = new DotEmitter(1000, 0, 0);
  this->emitter = new DotEmitter(200, 0, 0);
  this->emitter->setParent(this);
  this->emitter->setSpread(M_PI,M_PI);
  this->emitter->setInheritSpeed(this->velocity.len());
  this->emitter->setEmissionRate(500.0);
  this->emitter->setEmissionVelocity(this->velocity.len());

  this->mat = new Material("mBolt");
  //this->mat->setBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
  this->mat->setBlendFunc(GL_SRC_ALPHA,GL_ONE);
  this->mat->setDiffuse(1,1,1);
  this->mat->setDiffuseMap("laser_add.png");
  this->mat->setDiffuseMap("laser.png",1);

  dynamic_cast<StaticModel*>(this->getModel())->addMaterial(this->mat);
  dynamic_cast<StaticModel*>(this->getModel())->finalize();
}


/**
 *  standard deconstructor
 *
 */
MBolt::~MBolt ()
{
  // delete this->emitter;

  if (MBolt::trailParticles != NULL && MBolt::objectList().size() <= 1)
  {
    if (ParticleSystem::objectList().exists(MBolt::trailParticles))
      delete MBolt::trailParticles;
    MBolt::trailParticles = NULL;
  }
  if (MBolt::explosionParticles != NULL && MBolt::objectList().size() <= 1)
  {
    if (ParticleSystem::objectList().exists(MBolt::explosionParticles))
      delete MBolt::explosionParticles;
    MBolt::explosionParticles = NULL;
  }
}

SpriteParticles* MBolt::trailParticles = NULL;
SpriteParticles* MBolt::explosionParticles = NULL;

void MBolt::activate()
{
  if (unlikely(MBolt::trailParticles == NULL))
  {
    MBolt::trailParticles = new SpriteParticles(1000);
    MBolt::trailParticles->setName("BoomerangProjectileTrailParticles");
    MBolt::trailParticles->setMaterialTexture("maps/radial-trans-noise.png");
    MBolt::trailParticles->setLifeSpan(0.3, 0);
    MBolt::trailParticles->setRadius(0.0, .8);
    MBolt::trailParticles->setRadius(1.0, .2);
    MBolt::trailParticles->setColor(0.0, 1,0,0,.9);
    MBolt::trailParticles->setColor(0.2, .8,.2,0,.9);
    MBolt::trailParticles->setColor(0.5, .8,.4,.4,.8);
    MBolt::trailParticles->setColor(1.0, .8,.8,.8,.7);
  }
  if (unlikely(MBolt::explosionParticles == NULL))
  {
    MBolt::explosionParticles = new SpriteParticles(1000);
    MBolt::explosionParticles->setName("MBoltExplosionParticles");
    MBolt::explosionParticles->setLifeSpan(.5, .3);
    MBolt::explosionParticles->setRadius(0.0, 10.0);
    MBolt::explosionParticles->setRadius(.5, 6.0);
    MBolt::explosionParticles->setRadius(1.0, 3.0);
    MBolt::explosionParticles->setColor(0.0, 1,1,0,.9);
    MBolt::explosionParticles->setColor(0.5, .8,.8,0,.5);
    MBolt::explosionParticles->setColor(1.0, .8,.8,.7,.0);
  }

  this->setDamage(10);
  this->setHealth(0);

  this->emitter->setSystem(MBolt::trailParticles);

  this->emitter->setSpread(0);
  this->emitter->setEmissionRate(10.0);
  this->emitter->setEmissionVelocity(0);
  this->updateNode(0);
}


void MBolt::deactivate()
{
  assert (MBolt::trailParticles != NULL);
  MBolt::trailParticles->removeEmitter(this->emitter);

  assert (MBolt::explosionParticles != NULL);
  MBolt::explosionParticles->removeEmitter(this->emitter);
  this->lifeCycle = 0.0;

  this->toList(OM_NULL);
  this->removeNode();
  MBolt::fastFactory->kill(this);
}


void MBolt::collidesWith(WorldEntity* entity, const Vector& location)
{
  if (this->hitEntity != entity && entity->isA(CL_NPC))
    this->destroy( entity );
  this->hitEntity = entity;
  dynamic_cast<SpaceShip*>(entity)->damage(this->getDamage(),0);
}

/**
 *  signal tick, time dependent things will be handled here
 * @param dt time since last tick
*/
void MBolt::tick (float dt)
{
  //Vector v = *this->flightDirection * ( this->speed * time * 1000 + 0.1);
  Vector v = this->velocity * dt;
  this->shiftCoor(v);

  if (this->tickLifeCycle(dt))
    this->deactivate();

  this->angle += MBolt::rotationSpeed * dt;
}

/**
 *  the function gets called, when the projectile is destroyed
*/
void MBolt::destroy (WorldEntity* killer)
{
  Projectile::destroy( killer );
  PRINTF(5)("DESTROY MBolt\n");
  this->lifeCycle = .95; //!< @todo calculate this usefully.

  this->emitter->setSystem(MBolt::explosionParticles);
}


void MBolt::draw () const
{
  glPushAttrib(GL_ENABLE_BIT);
  glDisable(GL_LIGHTING);

  glPushMatrix();
  float matrix[4][4];
  glTranslatef (this->getAbsCoor ().x, this->getAbsCoor ().y, this->getAbsCoor ().z);
  glRotatef(this->angle, 1.0f, 0.0f, 0.0f); //spinning missile
  this->getAbsDir().matrix (matrix);
  glMultMatrixf((float*)matrix);

  glScalef(0.75, 0.7/4, 0.7/4);  // no double rescale

  this->mat->select();
  dynamic_cast<StaticModel*>(this->getModel())->draw();
  this->mat->unselect();
  glPopMatrix();
  glPopAttrib();


}

